<?php
/**
 * Plugin Name: FeedCanvas
 * Description: Display recent updates from external RSS feeds in a stylish list format. Built for mutual RSS promotion.
 * Version: 1.32
 * Author: luminobank
 * Author URI: https://luminobank.com/
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: feedcanvas
 */

// feedcanvas-default.php に定義された初期設定値を読み込む関数
function feedcanvas_load_default_values() {
    $default_file = plugin_dir_path(__FILE__) . 'feedcanvas-default.php';
    if (file_exists($default_file)) {
        return include $default_file;
    }
    return [];
}

// 「デフォルトに戻す」ボタンが押されたときのみ、設定値をfeedcanvas-default.phpに基づき初期化する処理
add_action('admin_init', function () {
    if (isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'POST' &&
        isset($_POST['reset_button']) && // ← Resetボタンからのみ反応
        isset($_POST['feedcanvas_reset_defaults']) &&
        check_admin_referer('rss_display_reset_defaults_action')
    ) {
        $default_options = feedcanvas_load_default_values(); // feedcanvas-default.phpからデフォルト値読み込み
        foreach ($default_options as $key => $value) {
            update_option($key, $value);
        }

        // 管理画面に戻る（「reset=1」で通知表示対応）
        wp_redirect(add_query_arg([
            'page' => 'rss-display',
            'reset' => 1,
            '_wpnonce' => wp_create_nonce('rss_display_settings-options')
        ], admin_url('options-general.php')));
        exit;
    }
});

add_action('admin_menu', function () {
    add_options_page('FeedCanvas Settings', 'FeedCanvas', 'manage_options', 'rss-display', 'rss_display_settings_page');
});

// 変更を保存ボタンが押された時に、表示しいてるタブの表示を維持する
add_action('admin_enqueue_scripts', function ($hook) {
    if ($hook === 'settings_page_rss-display') {
        $plugin_data = get_file_data(__FILE__, ['Version' => 'Version']);
        wp_enqueue_script(
            'rss-display-tabs-js',
            plugin_dir_url(__FILE__) . 'admin-tabs.js',
            [],
            $plugin_data['Version'],
            true
        );
    }
});

add_action('wp_enqueue_scripts', function () {
    $plugin_data = get_file_data(__FILE__, ['Version' => 'Version']);
    wp_enqueue_style(
        'feedcanvas-style',
        plugin_dir_url(__FILE__) . 'style.css',
        [],
        $plugin_data['Version']
    );
});

// 保存用ファイルパス
define('FEEDCANVAS_SAVED_FILE', plugin_dir_path(__FILE__) . 'feedcanvas-saved.php');

// 「設定を保存する」処理
add_action('admin_init', function () {
    if (isset($_POST['feedcanvas_save_settings']) &&
        check_admin_referer('rss_display_save_settings_action')) {

        $default_values = feedcanvas_load_default_values();
        $option_keys = array_keys($default_values); // すべての既知キーを取得
        $saved = [];

        foreach ($option_keys as $key) {
            $value = get_option($key);

            // フィードURL配列は50件に揃える（空文字補完）
            if ($key === 'rss_display_feeds') {
                if (!is_array($value)) {
                    $value = [];
                }
                $value = array_values(array_map('sanitize_text_field', $value));
                $value = array_pad($value, 50, "");
            }

            // 数値っぽい文字列を整数に変換（デフォルト形式と合わせる）
            if (is_string($value) && is_numeric($value) && ctype_digit($value)) {
                $value = intval($value);
            }

            $saved[$key] = $value;
        }

        $file_path = plugin_dir_path(__FILE__) . 'feedcanvas-saved.php';

        $header = "<?php\n// 保存された設定値（FeedCanvas用）\n// feedcanvas-saved.php\n\n";
        // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_var_export -- safe usage for settings export
        $export_code = $header . 'return ' . var_export($saved, true) . ";\n";


        file_put_contents($file_path, $export_code);

        wp_redirect(add_query_arg([
            'page' => 'rss-display',
            'saved' => 1,
            '_wpnonce' => wp_create_nonce('rss_display_settings-options')
        ], admin_url('options-general.php')));
        exit;
    }
});

// 「設定を読み込む」処理
add_action('admin_init', function () {
    if (isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'POST' &&
        isset($_POST['feedcanvas_load_settings']) &&
        check_admin_referer('rss_display_load_settings_action') &&
        file_exists(FEEDCANVAS_SAVED_FILE) &&
        is_readable(FEEDCANVAS_SAVED_FILE)
    ) {
        $saved_options = include FEEDCANVAS_SAVED_FILE;

        if (is_array($saved_options)) {
            // feedcanvas-default.php のキーに限定して上書き
            $valid_keys = array_keys(feedcanvas_load_default_values());

            foreach ($valid_keys as $key) {
                if (array_key_exists($key, $saved_options)) {
                    update_option($key, $saved_options[$key]);
                }
            }
        }

        wp_redirect(add_query_arg([
            'page' => 'rss-display',
            'loaded' => 1,
            '_wpnonce' => wp_create_nonce('rss_display_settings-options')
        ], admin_url('options-general.php')));
        exit;
    }
});

add_action('admin_init', function () {
    register_setting('rss_display_settings', 'rss_display_language', ['sanitize_callback' => function($val) {return in_array($val, ['en', 'ja']) ? $val : 'en';}]);
    register_setting('rss_display_settings', 'rss_display_feeds', ['sanitize_callback' => 'rss_display_feeds_sanitize_pure']);
    // List Design 1 の設定
    register_setting('rss_display_settings', 'rss_display_list_header_text', ['sanitize_callback' => 'rss_display_list_header_text_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_scroll_height', ['sanitize_callback' => 'rss_display_list_scroll_height_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_bg_color', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_hover_bg_color', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_link_color', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_link_hover_color', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_line_spacing', ['sanitize_callback' => 'rss_display_list_line_spacing_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_border_enabled', ['sanitize_callback' => function ($val) {return ($val === 'yes' || $val === 'no') ?$val : 'no';}]);
    register_setting('rss_display_settings', 'rss_display_list_border_color', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_font_size', ['sanitize_callback' => 'rss_display_list_font_size_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_order', ['sanitize_callback' => function($val) { return in_array($val, ['input', 'date']) ? $val : 'date'; }]);
    register_setting('rss_display_settings', 'rss_display_list_outline_color', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_outline_width', ['sanitize_callback' => 'rss_display_list_outline_width_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_outline_radius', ['sanitize_callback' => 'rss_display_list_outline_radius_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_top', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_right', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_bottom', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_left', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_header_left', ['sanitize_callback' => 'rss_display_header_left_sanitize']);
    register_setting('rss_display_settings', 'rss_display_header_font_size', ['sanitize_callback' => 'rss_display_header_font_size_sanitize']);
    register_setting('rss_display_settings', 'rss_display_header_text_color', ['sanitize_callback' => 'sanitize_hex_color']);
    // List Design 2 の設定
    register_setting('rss_display_settings', 'rss_display_list_header_text_2', ['sanitize_callback' => 'rss_display_list_header_text_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_scroll_height_2', ['sanitize_callback' => 'rss_display_list_scroll_height_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_bg_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_hover_bg_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_link_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_link_hover_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_line_spacing_2', ['sanitize_callback' => 'rss_display_list_line_spacing_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_border_enabled_2', ['sanitize_callback' => function ($val) {return ($val === 'yes' || $val === 'no') ?$val : 'no';}]);
    register_setting('rss_display_settings', 'rss_display_list_border_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_font_size_2', ['sanitize_callback' => 'rss_display_list_font_size_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_order_2', ['sanitize_callback' => function($val) { return in_array($val, ['input', 'date']) ? $val : 'date'; }]);
    register_setting('rss_display_settings', 'rss_display_list_outline_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
    register_setting('rss_display_settings', 'rss_display_list_outline_width_2', ['sanitize_callback' => 'rss_display_list_outline_width_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_outline_radius_2', ['sanitize_callback' => 'rss_display_list_outline_radius_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_top_2', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_right_2', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_bottom_2', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_list_margin_left_2', ['sanitize_callback' => 'rss_display_box_margin_sanitize']);
    register_setting('rss_display_settings', 'rss_display_header_left_2', ['sanitize_callback' => 'rss_display_header_left_sanitize']);
    register_setting('rss_display_settings', 'rss_display_header_font_size_2', ['sanitize_callback' => 'rss_display_header_font_size_sanitize']);
    register_setting('rss_display_settings', 'rss_display_header_text_color_2', ['sanitize_callback' => 'sanitize_hex_color']);
});

// 設定を保存する処理（feedcanvas-saved.php に現在の設定を保存）
add_action('admin_init', function () {
    if (isset($_POST['feedcanvas_save_settings']) &&
        check_admin_referer('rss_display_save_settings_action')) {

        $option_keys = array_keys(feedcanvas_load_default_values()); // すべての既知キーを取得
        $saved = [];

        foreach ($option_keys as $key) {
            $saved[$key] = get_option($key);
        }

        $file_path = plugin_dir_path(__FILE__) . 'feedcanvas-saved.php';
        // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_var_export -- used for structured config export
        $export_code = "<?php\n// 保存されたFeedCanvasの設定値\nreturn " . var_export($saved, true) . ";\n";

        file_put_contents($file_path, $export_code);

        // 管理画面にリダイレクトして通知
        wp_redirect(add_query_arg([
            'page' => 'rss-display',
            'saved' => 1,
            '_wpnonce' => wp_create_nonce('rss_display_settings-options')
        ], admin_url('options-general.php')));
        exit;
    }
});

// 設定を読み込む処理（feedcanvas-saved.php から設定を読み込んで反映）
add_action('admin_init', function () {
    if (isset($_POST['feedcanvas_load_settings']) &&
        check_admin_referer('rss_display_load_settings_action')) {

        $saved_file = plugin_dir_path(__FILE__) . 'feedcanvas-saved.php';
        if (file_exists($saved_file)) {
            $saved_options = include $saved_file;
            if (is_array($saved_options)) {
                foreach ($saved_options as $key => $value) {
                    update_option($key, $value);
                }
            }
        }

        // 管理画面にリダイレクトして通知（通知は必要に応じて後で追加）
        wp_redirect(add_query_arg([
            'page' => 'rss-display',
            'loaded' => 1,
            '_wpnonce' => wp_create_nonce('rss_display_settings-options')
        ], admin_url('options-general.php')));
        exit;
    }
});

// プラグイン初回有効時にデフォルト値をセットする
register_activation_hook(__FILE__, 'feedcanvas_set_initial_defaults');

function feedcanvas_set_initial_defaults() {
    $defaults = feedcanvas_load_default_values();
    foreach ($defaults as $key => $value) {
        if (get_option($key) === false) {
            update_option($key, $value);
        }
    }
}

// RSS feed URLの入力で、空白を除去し、重複を排除する
function rss_display_feeds_sanitize_pure($feeds) {
    if (!is_array($feeds)) return [];
    $feeds = array_filter(array_map('trim', $feeds));
    $feeds = array_unique($feeds);
    return array_values(array_map('sanitize_text_field', $feeds));
}

function rss_display_list_header_text_sanitize($text) {
    $text = sanitize_text_field($text);
    return mb_strlen($text) > 20 ? mb_substr($text, 0, 20) : $text;
}

function rss_display_list_scroll_height_sanitize($value) {
    $h = intval($value);
    return max(40, min(2000, $h));
}

function rss_display_list_line_spacing_sanitize($value) {
    return max(0, min(30, intval($value)));
}

function rss_display_list_font_size_sanitize($value) {
    $v = intval($value);
    return max(2, min(40, $v)); // 2〜40px にクリッピング
}

function rss_display_list_outline_width_sanitize($value) {
    $v = intval($value);
    return max(0, min(10, $v)); // 0〜10px に制限
}

function rss_display_list_outline_radius_sanitize($value) {
    $v = intval($value);
    return max(0, min(10, $v)); // 0〜10px に制限
}

function rss_display_list_order_sanitize($val) {
    return in_array($val, ['input', 'date']) ? $val : 'date';
}

function rss_display_box_margin_sanitize($value) {
    return max(0, min(100, intval($value)));
}

function rss_display_header_left_sanitize($value) {
    return max(0, min(100, intval($value)));
}

function rss_display_header_font_size_sanitize($value) {
    return max(2, min(40, intval($value)));
}

add_action('update_option_rss_display_feeds', function ($old_value, $new_value) {
    require_once ABSPATH . WPINC . '/feed.php';
    $site_names = [];
    foreach ($new_value as $url) {
        $rss = fetch_feed($url);
        $site_names[] = (!is_wp_error($rss) && $rss->get_title()) ? sanitize_text_field($rss->get_title()) : '';
    }
    update_option('rss_display_site_names', $site_names);
}, 10, 2);

function rss_display_settings_page() {
    $lang = get_option('rss_display_language', 'en');
    $feeds = get_option('rss_display_feeds', []);
    $site_names = get_option('rss_display_site_names', []);

    $list_order = get_option('rss_display_list_order', 'date');
    $list_header_text = get_option('rss_display_list_header_text', '他サイト更新情報');
    $scroll_height = get_option('rss_display_list_scroll_height', 200);
    $bg_color = get_option('rss_display_list_bg_color', '#ffffff');
    $hover_bg_color = get_option('rss_display_list_hover_bg_color', '#f0f0f0');
    $link_color = get_option('rss_display_list_link_color', '#333333');
    $link_hover_color = get_option('rss_display_list_link_hover_color', '#000000');
    $line_spacing = get_option('rss_display_list_line_spacing', 8);
    $border_enabled = get_option('rss_display_list_border_enabled', 'yes');
    $border_color = get_option('rss_display_list_border_color', '#cccccc');
    $list_font_size = get_option('rss_display_list_font_size', 16);
    $outline_color = get_option('rss_display_list_outline_color', '#333333'); // デフォルトの囲み線色
    $outline_width = get_option('rss_display_list_outline_width', 1);          // デフォルト1px
    $outline_radius = get_option('rss_display_list_outline_radius', 0);        // デフォルト0px
    $box_margin_top = max(0, min(100, intval(get_option('rss_display_list_box_margin_top', 0))));
    $box_margin_right = max(0, min(100, intval(get_option('rss_display_list_box_margin_right', 0))));
    $box_margin_bottom = max(0, min(100, intval(get_option('rss_display_list_box_margin_bottom', 20))));
    $box_margin_left = max(0, min(100, intval(get_option('rss_display_list_box_margin_left', 0))));
    $header_left = get_option('rss_display_header_left', 40);
    $header_font_size = get_option('rss_display_header_font_size', 12);
    $header_text_color = get_option('rss_display_header_text_color', '#333333');

    $list_order_2 = get_option('rss_display_list_order_2', 'date');
    $list_header_text_2 = get_option('rss_display_list_header_text_2', '他サイト更新情報');
    $scroll_height_2 = get_option('rss_display_list_scroll_height_2', 200);
    $bg_color_2 = get_option('rss_display_list_bg_color_2', '#ffffff');
    $hover_bg_color_2 = get_option('rss_display_list_hover_bg_color_2', '#f0f0f0');
    $link_color_2 = get_option('rss_display_list_link_color_2', '#333333');
    $link_hover_color_2 = get_option('rss_display_list_link_hover_color_2', '#000000');
    $line_spacing_2 = get_option('rss_display_list_line_spacing_2', 8);
    $border_enabled_2 = get_option('rss_display_list_border_enabled_2', 'yes');
    $border_color_2 = get_option('rss_display_list_border_color_2', '#cccccc');
    $list_font_size_2 = get_option('rss_display_list_font_size_2', 16);
    $outline_color_2 = get_option('rss_display_list_outline_color_2', '#333333');
    $outline_width_2 = get_option('rss_display_list_outline_width_2', 1);
    $outline_radius_2 = get_option('rss_display_list_outline_radius_2', 0);
    $box_margin_top_2 = max(0, min(100, intval(get_option('rss_display_list_box_margin_top_2', 0))));
    $box_margin_right_2 = max(0, min(100, intval(get_option('rss_display_list_box_margin_right_2', 0))));
    $box_margin_bottom_2 = max(0, min(100, intval(get_option('rss_display_list_box_margin_bottom_2', 20))));
    $box_margin_left_2 = max(0, min(100, intval(get_option('rss_display_list_box_margin_left_2', 0))));
    $header_left_2 = get_option('rss_display_header_left_2', 40);
    $header_font_size_2 = get_option('rss_display_header_font_size_2', 12);
    $header_text_color_2 = get_option('rss_display_header_text_color_2', '#333333');
?>
<div class="wrap">
    <?php $lang = get_option('rss_display_language', 'en'); ?>
    <h1><?php echo $lang === 'ja' ? 'FeedCanvas 設定' : 'FeedCanvas Settings' ?></h1>
    
    <h2 class="nav-tab-wrapper">
        <a href="#tab-general" class="nav-tab"><?php echo ($lang === 'ja') ? 'General' : 'General' ?></a>
        <a href="#tab-feed-urls" class="nav-tab"><?php echo ($lang === 'ja') ? 'RSS feed URL' : 'RSS feed URL' ?></a>
        <a href="#tab-list-design1" class="nav-tab"><?php echo ($lang === 'ja') ? 'リスト表示デザイン1' : 'List Design 1' ?></a>
        <a href="#tab-list-design2" class="nav-tab"><?php echo ($lang === 'ja') ? 'リスト表示デザイン2' : 'List Design 2' ?></a>
        <a href="#tab-tile-design" class="nav-tab"><?php echo ($lang === 'ja') ? 'タイル表示デザイン' : 'Tile Design' ?></a>
    </h2>

    <form method="post" action="options.php">

        <?php if (
            isset($_GET['reset']) &&
            sanitize_text_field(wp_unslash($_GET['reset'])) == 1 &&
            isset($_GET['_wpnonce']) &&
            wp_verify_nonce(sanitize_text_field(wp_unslash($_GET['_wpnonce'])), 'rss_display_settings-options')
        ): ?>
        <div class="updated notice is-dismissible">
            <p><?php echo ($lang === 'ja') ? 'すべての設定をデフォルトに戻しました。' : 'All settings have been reset to default values.' ?></p>
        </div>
        <?php endif; ?>

        <?php if (
            isset($_GET['saved']) &&
            sanitize_text_field(wp_unslash($_GET['saved'])) == 1 &&
            isset($_GET['_wpnonce']) &&
            wp_verify_nonce(sanitize_text_field(wp_unslash($_GET['_wpnonce'])), 'rss_display_settings-options')
        ): ?>
            <div class="updated notice is-dismissible">
                <p><?php echo ($lang === 'ja') ? '現在の設定を保存しました。' : 'Current settings have been saved.' ?></p>
            </div>
        <?php endif; ?>

        <?php if (
            isset($_GET['loaded']) &&
            sanitize_text_field(wp_unslash($_GET['loaded'])) == 1 &&
            isset($_GET['_wpnonce']) &&
            wp_verify_nonce(sanitize_text_field(wp_unslash($_GET['_wpnonce'])), 'rss_display_settings-options')
        ): ?>
            <div class="updated notice is-dismissible">
                <p><?php echo ($lang === 'ja') ? '保存された設定を読み込みました。' : 'Saved settings have been loaded.' ?></p>
            </div>
        <?php endif; ?>

        <?php settings_fields('rss_display_settings'); ?>
        <input type="hidden" name="rss_display_last_tab" id="rss_display_last_tab" value="" />

        <div id="tab-general" class="rss-tab-content" style="display:block;">
            <table class="form-table">
                <tr>
                    <th>Language</th>
                    <td>
                        <label><input type="radio" name="rss_display_language" value="en" <?php echo $lang === 'en' ? 'checked' : '' ?> />English / 英語</label><br />
                        <label><input type="radio" name="rss_display_language" value="ja" <?php echo $lang === 'ja' ? 'checked' : '' ?> />Japanese / 日本語</label>
                    </td>
                </tr>

                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>

                <tr>
                    <th><?php echo ($lang === 'ja') ? '設定ファイル操作' : 'Settings File'; ?></th>
                    <td>
                        <button type="button"
                            class="button"
                            style="margin-right:10px;"
                            onclick="saveCurrentSettings();">
                            <?php echo ($lang === 'ja') ? '設定を保存する' : 'Save Settings'; ?>
                        </button>
                        <button type="button"
                            class="button"
                            onclick="loadSavedSettings();">
                            <?php echo ($lang === 'ja') ? '設定を読み込む' : 'Load Settings'; ?>
                        </button>
                        <p class="description"><?php echo ($lang === 'ja') ? '現在の設定値を保存・復元できます。' : 'Save and restore your current settings.'; ?></p>
                    </td>
                </tr>

                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>

                <tr>
                    <th><?php echo ($lang === 'ja') ? '設定初期化' : 'Reset Settings'; ?></th>
                    <td>
                        <button type="button"
                            class="button button-delete"
                            style="border: 1px solid red; color: red; background-color: #fff;"
                            onclick="resetToDefaults();">
                            <?php echo ($lang === 'ja') ? 'デフォルトに戻す' : 'Reset to Defaults'; ?>
                        </button>
                        <p class="description"><?php echo ($lang === 'ja') ? '【注意】全ての設定値をデフォルトに戻します。' : '[Notice] Resets all settings to default values.'; ?></p>
                    </td>
                </tr>

                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>

                <tr>
                    <th><?php echo $lang === 'ja' ? 'このプラグインについて' : 'About this plugin' ?></th>
                    <td>
                        <?php if ($lang === 'ja'): ?>
                            FeedCanvasプラグインは相互RSSの仕組みを活用して他サイトとの間でアクセストレードを促進することを目指すものです。詳細な仕様や使い方は
                            <a href="https://luminobank.com/?p=788" target="_blank" rel="noopener noreferrer">https://luminobank.com/?p=788</a> にて公開しております。
                        <?php else: ?>
                            The FeedCanvas plugin aims to promote access trading with other sites using reciprocal RSS feeds. For full documentation and usage details, visit
                            <a href="https://luminobank.com/?p=788" target="_blank" rel="noopener noreferrer">https://luminobank.com/?p=788</a>.
                        <?php endif; ?>
                    </td>
                </tr>

                <tr>
                    <th><?php echo $lang === 'ja' ? '更新履歴' : 'Revision history' ?></th>
                    <td>
                            <table style="width:auto; border-collapse:collapse;">
                                <thead>
                                    <tr>
                                        <th style="text-align:left; border-bottom: 1px solid #ccc; white-space: nowrap;">Version</th>
                                        <th style="text-align:left; border-bottom: 1px solid #ccc; width: 400px; word-wrap: break-word; word-break: break-word;"> <?php echo $lang === 'ja' ? '内容' : 'Description' ?></th>
                                        <th style="text-align:left; border-bottom: 1px solid #ccc; white-space: nowrap;"> <?php echo $lang === 'ja' ? '日付' : 'Date' ?></th>
                                    </tr>
                                </thead>
                                <tbody>
                                    <tr>
                                        <td style="white-space: nowrap;">1.32</td>
                                        <td style="width: 300px; word-wrap: break-word; word-break: break-word;">
                                            <?php echo $lang === 'ja'
                                                ? '設定を保存する機能の不具合修正。'
                                                : 'Fixed a bug in the settings save function.'; ?>
                                        </td>
                                        <td style="white-space: nowrap;">28-May-2025</td>
                                    </tr>
                                    <tr>
                                        <td style="white-space: nowrap;">1.31</td>
                                        <td style="width: 300px; word-wrap: break-word; word-break: break-word;">
                                            <?php echo $lang === 'ja'
                                                ? 'スクロール高さが設定通りにならない不具合を修正。リスト表示のリンク文字色・ホバー文字色をユーザー設定可能に。その他、細かなコード修正。'
                                                : 'Fixed issue where scroll height was not applied correctly. Added user-customizable link and hover text colors for list display. Minor code improvements.'; ?>
                                        </td>
                                        <td style="white-space: nowrap;">28-May-2025</td>
                                    </tr>
                                    <tr>
                                        <td style="white-space: nowrap;">1.30</td>
                                        <td style="width: 300px; word-wrap: break-word; word-break: break-word;">
                                            <?php echo $lang === 'ja'
                                                ? '設定値を保存／読み出しする機能を追加。'
                                                : 'Added functionality to save and load all setting values.'; ?>
                                        </td>
                                        <td style="white-space: nowrap;">26-May-2025</td>
                                    </tr>
                                    <tr>
                                        <td style="white-space: nowrap;">1.20</td>
                                        <td style="width: 300px; word-wrap: break-word; word-break: break-word;">
                                            <?php echo $lang === 'ja'
                                                ? '全設定値をデフォルトに戻す機能を追加。'
                                                : 'Added a feature to reset all settings to default values.'; ?>
                                        </td>
                                        <td style="white-space: nowrap;">25-May-2025</td>
                                    </tr>
                                    <tr>
                                        <td style="white-space: nowrap;">1.10</td>
                                        <td style="width: 300px; word-wrap: break-word; word-break: break-word;">
                                            <?php echo $lang === 'ja'
                                                ? 'リスト表示を2種類に拡張（リストデザイン1 / リストデザイン2）し、用途に応じたカスタマイズが可能に。'
                                                : 'Expanded list display to support two styles: List Design 1 and List Design 2.' ?>
                                        </td>
                                        <td style="white-space: nowrap;">25-May-2025</td>
                                    </tr>
                                    <tr>
                                        <td style="white-space: nowrap;">1.00</td>
                                        <td style="width: 300px; word-wrap: break-word; word-break: break-word;">
                                            <?php echo $lang === 'ja'
                                                ? 'ショートコード出力に対応した他サイトRSSフィードのリスト表示機能を実装。'
                                                : 'Implemented list display feature for external RSS feeds via shortcode.'
                                            ?>
                                        </td>
                                        <td style="white-space: nowrap;">24-May-2025</td>
                                    </tr>
                                    <tr>
                                        <td colspan="2" style="padding-top: 8px; font-size: 13px;">
                                            <?php
                                            if ($lang === 'ja') {
                                                echo 'これ以前の更新履歴は <a href="https://luminobank.com/" target="_blank">https://luminobank.com/</a> を参照ください。';
                                            } else {
                                                echo 'For older update logs, please visit <a href="https://luminobank.com/" target="_blank">https://luminobank.com/</a>.';
                                            }
                                            ?>
                                        </td>
                                    </tr>
                                </tbody>
                            </table>
                    </td>
                </tr>

                <tr>
                    <th>Copyright</th>
                    <td>
                        FeedCanvas by <a href="https://luminobank.com/" target="_blank" rel="noopener noreferrer">luminobank</a> &copy; 2025
                    </td>
                </tr>
            </table>
        </div>

        <div id="tab-feed-urls" class="rss-tab-content" style="display:block;">
            <p class="description" style="margin: 8px 0 20px 0;">
                <?php echo $lang === 'ja'
                    ? 'RSS feedのURL入力を変更した後で「変更を保存」するとRSS feed URLの右側にサイト名を表示します。'
                    : 'After changing the RSS feed URL entries, click "Save Changes" to display the site name next to each feed URL.'
                ?>
            </p>
            <table class="form-table">
                <?php for ($i = 0; $i < 50; $i++): ?>
                    <tr>
                        <th>RSS Feed URL <?php echo esc_html($i + 1) ?></th>
                        <td>
                            <input type="url" name="rss_display_feeds[]" value="<?php echo esc_attr($feeds[$i] ?? '') ?>" size="50" />
                            <span style="margin-left:10px;color:#666;"><?php echo esc_html($site_names[$i] ?? '') ?></span>
                        </td>
                    </tr>
                <?php endfor; ?>
            </table>
        </div>

        <!-- List Design 1 設定フォーム -->
        <div id="tab-list-design1" class="rss-tab-content" style="display:none;">
            <p class="description" style="margin: 8px 0 20px 0;">
                <?php echo $lang === 'ja'
                    ? 'この設定に対応するショートコードは[feedcanvas type="list"]です。'
                    : 'The shortcode corresponding to this setting is [feedcanvas type="list"].' ?>
           </p>
            <table class="form-table">
                <tr>
                    <th><?php echo $lang === 'ja' ? '表示順' : 'Sort Order' ?></th>
                    <td>
                        <label>
                            <input type="radio" name="rss_display_list_order" value="date" <?php echo get_option('rss_display_list_order', 'date') === 'date' ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? '更新日時が新しい順' : 'Newest first' ?>
                        </label><br>
                        <label>
                            <input type="radio" name="rss_display_list_order" value="input" <?php echo get_option('rss_display_list_order', 'date') === 'input' ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? 'RSS feed URLタブで入力された順' : 'Order entered in RSS feed URL tab' ?>
                        </label>
                    </td>
                </tr>
		<tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出しテキスト' : 'Header Text' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_header_text" value="<?php echo esc_attr($list_header_text) ?>" maxlength="40" />
                        <p class="description"><?php echo $lang === 'ja' ? '最大20文字まで' : 'Up to 20 characters' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出しの左位置' : 'Header Left Position' ?></th>
                    <td>
                        <input type="number" name="rss_display_header_left" value="<?php echo esc_attr(get_option('rss_display_header_left', 40)) ?>" min="0" max="100" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜100px' : '0 – 100px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出しフォントサイズ' : 'Header Font Size' ?></th>
                    <td>
                        <input type="number" name="rss_display_header_font_size" value="<?php echo esc_attr(get_option('rss_display_header_font_size', 12)) ?>" min="2" max="40" />
                        <p class="description"><?php echo $lang === 'ja' ? '2〜40px' : '2 – 40px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出し文字色' : 'Header Text Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_header_text_color" value="<?php echo esc_attr(get_option('rss_display_header_text_color', '#333333')) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'スクロール高さ' : 'Scroll Height' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_scroll_height" value="<?php echo esc_attr($scroll_height) ?>" min="40" max="2000" />
                        <p class="description"><?php echo $lang === 'ja' ? '40〜2,000px' : '40 – 2,000px.' ?></p>
                   </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '行間の余白' : 'Line Spacing' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_line_spacing" value="<?php echo esc_attr($line_spacing) ?>" min="0" max="30" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜30px' : '0 - 30px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '行間罫線' : 'Line Separator' ?></th>
                    <td>
                        <label>
                            <input type="radio" name="rss_display_list_border_enabled" value="yes" <?php echo ($border_enabled === 'yes') ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? '有り' : 'Enabled' ?>
                        </label>
                        <label>
                            <input type="radio" name="rss_display_list_border_enabled" value="no" <?php echo ($border_enabled === 'no') ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? '無し' : 'Disabled' ?>
                        </label>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '罫線の色' : 'Line Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_border_color" value="<?php echo esc_attr($border_color) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '背景色' : 'Background Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_bg_color" value="<?php echo esc_attr($bg_color) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'ホバー時背景色' : 'Hover Background Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_hover_bg_color" value="<?php echo esc_attr($hover_bg_color) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'テキストのフォントサイズ' : 'Text Font Size' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_font_size" value="<?php echo esc_attr($list_font_size) ?>" min="2" max="40" />
                        <p class="description"><?php echo $lang === 'ja' ? '2〜40px' : '2 – 40px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'リンク文字色' : 'Link Text Color'; ?></th>
                    <td>
                        <input type="text" name="rss_display_list_link_color" value="<?php echo esc_attr(get_option('rss_display_list_link_color', '#333333')) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff'; ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'リンクホバー色' : 'Link Hover Color'; ?></th>
                    <td>
                        <input type="text" name="rss_display_list_link_hover_color" value="<?php echo esc_attr(get_option('rss_display_list_link_hover_color', '#000000')) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff'; ?></p>
                    </td>
                </tr>
                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の色' : 'Outline Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_outline_color" value="<?php echo esc_attr($outline_color) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の太さ' : 'Outline Width' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_outline_width" value="<?php echo esc_attr($outline_width) ?>" min="0" max="10" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜10px' : '0 – 10px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の角の丸み' : 'Border Radius' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_outline_radius" value="<?php echo esc_attr($outline_radius) ?>" min="0" max="10" />
                        <p class="description"><?php echo $lang === 'ja' ? '0で直角、1〜10でラウンド' : '0 = square, 1–10 = round' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の外側余白' : 'Outer Margin' ?></th>
                    <td>
                        <?php echo $lang === 'ja' ? '上' : 'Top' ?>：
                        <input type="number" name="rss_display_list_margin_top" value="<?php echo esc_attr(get_option('rss_display_list_margin_top', 0)) ?>" min="0" max="100" style="width:60px;" />　
                        <?php echo $lang === 'ja' ? '右' : 'Right' ?>：
                        <input type="number" name="rss_display_list_margin_right" value="<?php echo esc_attr(get_option('rss_display_list_margin_right', 0)) ?>" min="0" max="100" style="width:60px;" />　
                        <?php echo $lang === 'ja' ? '下' : 'Bottom' ?>：
                        <input type="number" name="rss_display_list_margin_bottom" value="<?php echo esc_attr(get_option('rss_display_list_margin_bottom', 20)) ?>" min="0" max="100" style="width:60px;" />　
                        <?php echo $lang === 'ja' ? '左' : 'Left' ?>：
                        <input type="number" name="rss_display_list_margin_left" value="<?php echo esc_attr(get_option('rss_display_list_margin_left', 0)) ?>" min="0" max="100" style="width:60px;" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜100px' : '0 – 100px' ?></p>
                    </td>
                </tr>
            </table>
        </div>
 
        <!-- List Design 2 設定フォーム -->
        <div id="tab-list-design2" class="rss-tab-content" style="display:none;">
            <p class="description" style="margin: 8px 0 20px 0;">
                <?php echo $lang === 'ja'
                    ? 'この設定に対応するショートコードは[feedcanvas type="list2"]です。'
                    : 'The shortcode corresponding to this setting is [feedcanvas type="list2"].' ?>
           </p>
            <table class="form-table">
                <tr>
                    <th><?php echo $lang === 'ja' ? '表示順' : 'Sort Order' ?></th>
                    <td>
                        <label>
                            <input type="radio" name="rss_display_list_order_2" value="date" <?php echo get_option('rss_display_list_order_2', 'date') === 'date' ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? '更新日時が新しい順' : 'Newest first' ?>
                        </label><br>
                        <label>
                            <input type="radio" name="rss_display_list_order_2" value="input" <?php echo get_option('rss_display_list_order_2', 'date') === 'input' ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? 'RSS feed URLタブで入力された順' : 'Order entered in RSS feed URL tab' ?>
                        </label>
                    </td>
                </tr>
		<tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出しテキスト' : 'Header Text' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_header_text_2" value="<?php echo esc_attr($list_header_text_2) ?>" maxlength="40" />
                        <p class="description"><?php echo $lang === 'ja' ? '最大20文字まで' : 'Up to 20 characters' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出しの左位置' : 'Header Left Position' ?></th>
                    <td>
                        <input type="number" name="rss_display_header_left_2" value="<?php echo esc_attr(get_option('rss_display_header_left_2', 40)) ?>" min="0" max="100" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜100px' : '0 – 100px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出しフォントサイズ' : 'Header Font Size' ?></th>
                    <td>
                        <input type="number" name="rss_display_header_font_size_2" value="<?php echo esc_attr(get_option('rss_display_header_font_size_2', 12)) ?>" min="2" max="40" />
                        <p class="description"><?php echo $lang === 'ja' ? '2〜40px' : '2 – 40px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '見出し文字色' : 'Header Text Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_header_text_color_2" value="<?php echo esc_attr(get_option('rss_display_header_text_color_2', '#333333')) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'スクロール高さ' : 'Scroll Height' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_scroll_height_2" value="<?php echo esc_attr($scroll_height_2) ?>" min="40" max="2000" />
                        <p class="description"><?php echo $lang === 'ja' ? '40〜2,000px' : '40 – 2,000px.' ?></p>
                   </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '行間の余白' : 'Line Spacing' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_line_spacing_2" value="<?php echo esc_attr($line_spacing_2) ?>" min="0" max="30" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜30px' : '0 - 30px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '行間罫線' : 'Line Separator' ?></th>
                    <td>
                        <label>
                            <input type="radio" name="rss_display_list_border_enabled_2" value="yes" <?php echo ($border_enabled_2 === 'yes') ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? '有り' : 'Enabled' ?>
                        </label>
                        <label>
                            <input type="radio" name="rss_display_list_border_enabled_2" value="no" <?php echo ($border_enabled_2 === 'no') ? 'checked' : '' ?> />
                            <?php echo $lang === 'ja' ? '無し' : 'Disabled' ?>
                        </label>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '罫線の色' : 'Line Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_border_color_2" value="<?php echo esc_attr($border_color_2) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '背景色' : 'Background Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_bg_color_2" value="<?php echo esc_attr($bg_color_2) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'ホバー時背景色' : 'Hover Background Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_hover_bg_color_2" value="<?php echo esc_attr($hover_bg_color_2) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'テキストのフォントサイズ' : 'Text Font Size' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_font_size_2" value="<?php echo esc_attr($list_font_size_2) ?>" min="2" max="40" />
                        <p class="description"><?php echo $lang === 'ja' ? '2〜40px' : '2 – 40px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'リンク文字色' : 'Link Text Color'; ?></th>
                    <td>
                        <input type="text" name="rss_display_list_link_color_2" value="<?php echo esc_attr(get_option('rss_display_list_link_color_2', '#333333')) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff'; ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? 'リンクホバー色' : 'Link Hover Color'; ?></th>
                    <td>
                        <input type="text" name="rss_display_list_link_hover_color_2" value="<?php echo esc_attr(get_option('rss_display_list_link_hover_color_2', '#000000')) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff'; ?></p>
                    </td>
                </tr>
                <tr><td colspan="2"><hr style="border: none; border-top: 1px solid #999; margin: 16px 0;"></td></tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の色' : 'Outline Color' ?></th>
                    <td>
                        <input type="text" name="rss_display_list_outline_color_2" value="<?php echo esc_attr($outline_color_2) ?>" />
                        <p class="description"><?php echo $lang === 'ja' ? '#000000〜#ffffff 形式' : 'Hex format, #000000 - #ffffff' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の太さ' : 'Outline Width' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_outline_width_2" value="<?php echo esc_attr($outline_width_2) ?>" min="0" max="10" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜10px' : '0 – 10px' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の角の丸み' : 'Border Radius' ?></th>
                    <td>
                        <input type="number" name="rss_display_list_outline_radius_2" value="<?php echo esc_attr($outline_radius_2) ?>" min="0" max="10" />
                        <p class="description"><?php echo $lang === 'ja' ? '0で直角、1〜10でラウンド' : '0 = square, 1–10 = round' ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php echo $lang === 'ja' ? '囲み線の外側余白' : 'Outer Margin' ?></th>
                    <td>
                        <?php echo $lang === 'ja' ? '上' : 'Top' ?>：
                        <input type="number" name="rss_display_list_margin_top_2" value="<?php echo esc_attr(get_option('rss_display_list_margin_top_2', 0)) ?>" min="0" max="100" style="width:60px;" />　
                        <?php echo $lang === 'ja' ? '右' : 'Right' ?>：
                        <input type="number" name="rss_display_list_margin_right_2" value="<?php echo esc_attr(get_option('rss_display_list_margin_right_2', 0)) ?>" min="0" max="100" style="width:60px;" />　
                        <?php echo $lang === 'ja' ? '下' : 'Bottom' ?>：
                        <input type="number" name="rss_display_list_margin_bottom_2" value="<?php echo esc_attr(get_option('rss_display_list_margin_bottom_2', 20)) ?>" min="0" max="100" style="width:60px;" />　
                        <?php echo $lang === 'ja' ? '左' : 'Left' ?>：
                        <input type="number" name="rss_display_list_margin_left_2" value="<?php echo esc_attr(get_option('rss_display_list_margin_left_2', 0)) ?>" min="0" max="100" style="width:60px;" />
                        <p class="description"><?php echo $lang === 'ja' ? '0〜100px' : '0 – 100px' ?></p>
                    </td>
                </tr>
            </table>
        </div>

        <div id="tab-tile-design" class="rss-tab-content" style="display:none;">
            <p>（タイル表示のデザイン設定は今後追加予定）</p>
        </div>

        <?php
        $lang = get_option('rss_display_language', 'en');
        $save_label = ($lang === 'ja') ? '変更を保存' : 'Save Changes';
        submit_button($save_label);
        ?>
    </form>
</div>

<script>
function resetToDefaults() {
    if (!confirm('<?php echo ($lang === 'ja') ? '本当にデフォルトに戻しますか？' : 'Are you sure you want to reset to default settings?'; ?>')) return;

    const form = document.createElement('form');
    form.method = 'POST';
    form.action = '';

    const input1 = document.createElement('input');
    input1.type = 'hidden';
    input1.name = 'feedcanvas_reset_defaults';
    input1.value = '1';
    form.appendChild(input1);

    const input2 = document.createElement('input');
    input2.type = 'hidden';
    input2.name = 'reset_button';
    input2.value = '1';
    form.appendChild(input2);

    // nonce hidden input
    const nonceInput = document.createElement('input');
    nonceInput.type = 'hidden';
    nonceInput.name = '_wpnonce';
    nonceInput.value = '<?php echo esc_js(wp_create_nonce("rss_display_reset_defaults_action")); ?>';
    form.appendChild(nonceInput);

    document.body.appendChild(form);
    form.submit();
}
</script>

<script>
function saveCurrentSettings() {
    if (!confirm('<?php echo ($lang === 'ja') ? '現在の設定を保存しますか？' : 'Do you want to save the current settings?'; ?>')) return;
    const form = document.createElement('form');
    form.method = 'POST';
    form.action = '';

    const input1 = document.createElement('input');
    input1.type = 'hidden';
    input1.name = 'feedcanvas_save_settings';
    input1.value = '1';
    form.appendChild(input1);

    const nonceInput = document.createElement('input');
    nonceInput.type = 'hidden';
    nonceInput.name = '_wpnonce';
    nonceInput.value = '<?php echo esc_js(wp_create_nonce("rss_display_save_settings_action")); ?>';
    form.appendChild(nonceInput);

    document.body.appendChild(form);
    form.submit();
}

function loadSavedSettings() {
    if (!confirm('<?php echo ($lang === 'ja') ? '保存された設定を読み込みますか？' : 'Do you want to load the saved settings?'; ?>')) return;
    const form = document.createElement('form');
    form.method = 'POST';
    form.action = '';

    const input1 = document.createElement('input');
    input1.type = 'hidden';
    input1.name = 'feedcanvas_load_settings';
    input1.value = '1';
    form.appendChild(input1);

    const nonceInput = document.createElement('input');
    nonceInput.type = 'hidden';
    nonceInput.name = '_wpnonce';
    nonceInput.value = '<?php echo esc_js(wp_create_nonce("rss_display_load_settings_action")); ?>';
    form.appendChild(nonceInput);

    document.body.appendChild(form);
    form.submit();
}
</script>

<script>
    (function () {
        const tabs = document.querySelectorAll('.nav-tab-wrapper a');
        const contents = document.querySelectorAll('.rss-tab-content');
        tabs.forEach(tab => {
            tab.addEventListener('click', function (e) {
                e.preventDefault();
                tabs.forEach(t => t.classList.remove('nav-tab-active'));
                this.classList.add('nav-tab-active');
                contents.forEach(c => c.style.display = (c.id === this.getAttribute('href').substring(1)) ? 'block' : 'none');
            });
        });
    })();
</script>
<?php
}

add_shortcode('feedcanvas', 'rss_display_shortcode');
function rss_display_shortcode($atts) {
    $atts = shortcode_atts(['type' => 'list'], $atts);
    $type = $atts['type'];
    $is_type2 = ($type === 'list2');

    $rss_urls = get_option('rss_display_feeds', []);
    require_once ABSPATH . WPINC . '/feed.php';
    $items = [];

    foreach ($rss_urls as $url) {
        if (!$url) continue;
        $rss = fetch_feed($url);
        if (!is_wp_error($rss)) {
            $rss_items = $rss->get_items(0, 1); // 各フィードから1件だけ取得
            if (!empty($rss_items)) {
                $items[] = $rss_items[0];
            }
        }
    }

    $order_option_name = $is_type2 ? 'rss_display_list_order_2' : 'rss_display_list_order';
    $order_type = get_option($order_option_name, 'date');
    if ($order_type === 'date') {
        usort($items, function ($a, $b) {
            return $b->get_date('U') - $a->get_date('U');
        });
    }

    $items = array_slice($items, 0, 50);

    $prefix = $is_type2 ? '_2' : '';  // list2の場合はオプション名に「_2」を付加

    $header = esc_html(get_option("rss_display_list_header_text$prefix", '他サイト更新情報'));
    $header_left = max(0, min(100, intval(get_option("rss_display_header_left$prefix", 10))));
    $header_font_size = max(2, min(40, intval(get_option("rss_display_header_font_size$prefix", 12))));
    $header_text_color = esc_attr(get_option("rss_display_header_text_color$prefix", '#333333'));
    $scroll_height = max(40, min(2000, intval(get_option("rss_display_list_scroll_height$prefix", 200))));
    $bg_color = esc_attr(get_option("rss_display_list_bg_color$prefix", '#ffffff'));
    $hover_bg_color = esc_attr(get_option("rss_display_list_hover_bg_color$prefix", '#eef6ff'));
    $list_font_size = max(2, min(40, intval(get_option("rss_display_list_font_size$prefix", 14))));
    $link_color = esc_attr(get_option("rss_display_list_link_color$prefix", '#333333'));
    $link_hover_color = esc_attr(get_option("rss_display_list_link_hover_color$prefix", '#000000'));
    $line_spacing = max(0, min(30, intval(get_option("rss_display_list_line_spacing$prefix", 8))));
    $outline_color  = esc_attr(get_option("rss_display_list_outline_color$prefix", '#333333'));
    $outline_width  = max(0, min(10, intval(get_option("rss_display_list_outline_width$prefix", 1))));
    $outline_radius = max(0, min(10, intval(get_option("rss_display_list_outline_radius$prefix", 0))));
    $border_enabled = get_option("rss_display_list_border_enabled$prefix", 'yes');
    $border_color = esc_attr(get_option("rss_display_list_border_color$prefix", '#cccccc'));
    $margin_top = max(0, min(100, intval(get_option("rss_display_list_margin_top$prefix", 0))));
    $margin_right = max(0, min(100, intval(get_option("rss_display_list_margin_right$prefix", 0))));
    $margin_bottom = max(0, min(100, intval(get_option("rss_display_list_margin_bottom$prefix", 20))));
    $margin_left = max(0, min(100, intval(get_option("rss_display_list_margin_left$prefix", 0))));
    $outline_css = "{$outline_width}px solid {$outline_color}";
    $border_css = ($border_enabled === 'yes') ? "1px solid $border_color" : "none";

    ob_start();
    ?>
    <?php
        $type = ($atts['type'] === 'list2') ? 'list' : $atts['type']; // type-list を常に出す
        $list_class = ($atts['type'] === 'list2') ? 'list2' : 'list1';
    ?>
    <div class="rss-display-container type-<?php echo esc_attr($type) ?> <?php echo esc_attr($list_class); ?>"
        style="
                --header-left: <?php echo esc_attr($header_left) ?>px;
                --rss-scroll-height: <?php echo esc_attr($scroll_height) ?>px;
                --header-font-size: <?php echo esc_attr($header_font_size) ?>px;
                --header-text-color: <?php echo esc_attr($header_text_color) ?>;
                --list-bg-color: <?php echo esc_attr($bg_color) ?>;
                --list-hover-bg-color: <?php echo esc_attr($hover_bg_color) ?>;
                --list-link-color: <?php echo esc_attr($link_color); ?>;
                --list-link-hover-color: <?php echo esc_attr($link_hover_color); ?>;
                --list-border-style: <?php echo esc_attr($border_css) ?>;
                --list-outline-style: <?php echo esc_attr($outline_css) ?>;
                --list-border-radius: <?php echo esc_attr($outline_radius) ?>px;
                --list-font-size: <?php echo esc_attr($list_font_size) ?>px;
                --list-box-margin-top: <?php echo esc_attr($margin_top) ?>px;
                --list-box-margin-right: <?php echo esc_attr($margin_right) ?>px;
                --list-box-margin-bottom: <?php echo esc_attr($margin_bottom) ?>px;
                --list-box-margin-left: <?php echo esc_attr($margin_left) ?>px;
	">
        <?php if ($atts['type'] === 'list' || $atts['type'] === 'list2'): ?>
            <div class="rss-list-box">
                <div class="rss-list-header"><?php echo esc_html($header) ?></div>
                <div class="rss-scroll-wrapper" style="max-height: <?php echo esc_attr($scroll_height) ?>px;">
                    <?php foreach ($items as $item): ?>
                        <div class="rss-item-list" style="margin-bottom: <?php echo esc_attr($line_spacing) ?>px;">
                            <a href="<?php echo esc_url($item->get_link()) ?>" target="_blank"><?php echo esc_html($item->get_title()) ?></a>
                        </div>
                    <?php endforeach; ?>
                </div>
            </div>
        <?php elseif ($atts['type'] === 'tile'): ?>
            <div class="rss-scroll-wrapper">
                <?php foreach ($items as $item): ?>
                    <?php
                    $title = esc_html($item->get_title());
                    $link = esc_url($item->get_link());
                    $thumbnail = '';
                    $enclosure = $item->get_enclosure();
                    if ($enclosure && $enclosure->get_thumbnail()) {
                        $thumbnail = esc_url($enclosure->get_thumbnail());
                    } elseif (preg_match('/<img[^>]+src=[\'"]([^\'"]+)[\'"]/', $item->get_content(), $m)) {
                        $thumbnail = esc_url($m[1]);
                    }
                    ?>
                    <div class="rss-item-tile">
                        <?php if ($thumbnail): ?>
                            <a href="<?php echo esc_url($link) ?>" target="_blank" class="rss-item-tile-img-link">
                                <!-- 外部RSS画像のため wp_get_attachment_image() は使用不可 -->
                                <img src="<?php echo esc_url($thumbnail) ?>" alt="<?php echo esc_attr($title) ?>" class="rss-item-tile-img" />
                            </a>
                        <?php endif; ?>
                        <a href="<?php echo esc_url($link) ?>" target="_blank" class="rss-item-tile-title-link"><?php echo esc_html($title) ?></a>
                    </div>
                <?php endforeach; ?>
            </div>
        <?php endif; ?>
    </div>
    <?php
    return ob_get_clean();
}
?>